@@ -489,11 +489,11 @@ func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModif
 			if _, ok := views[view]; !ok {
 				views[view] = make(map[span.URI]*fileChange)
 			}
-			fh, err := s.GetFile(ctx, c.URI)
+			fh, err := s.ReadFile(ctx, c.URI)
 			if err != nil {
 				return nil, nil, err
 			}
-			content, err := fh.Read()
+			content, err := fh.Content()
 			if err != nil {
 				// Ignore the error: the file may be deleted.
 				content = nil
@@ -684,11 +684,11 @@ func (fs *overlayFS) updateOverlays(ctx context.Context, changes []source.FileMo
 			}
 			sameContentOnDisk = true
 		default:
-			fh, err := fs.delegate.GetFile(ctx, c.URI)
+			fh, err := fs.delegate.ReadFile(ctx, c.URI)
 			if err != nil {
 				return err
 			}
-			_, readErr := fh.Read()
+			_, readErr := fh.Content()
 			sameContentOnDisk = (readErr == nil && fh.FileIdentity().Hash == hash)
 		}
 		o = &Overlay{
