@@ -697,10 +697,10 @@ var BaseTemplate = template.Must(template.New("").Parse(`
 td.value {
   text-align: right;
 }
-ul.events {
-	list-style-type: none;
+ul.spans {
+	font-family: monospace;
+	font-size:   85%;
 }
-
 </style>
 {{block "head" .}}{{end}}
 </head>
@@ -904,6 +904,6 @@ var FileTmpl = template.Must(template.Must(BaseTemplate.Clone()).Parse(`
 	Kind: <b>{{.Kind}}</b><br>
 {{end}}
 <h3>Contents</h3>
-<pre>{{fcontent .Read}}</pre>
+<pre>{{fcontent .Content}}</pre>
 {{end}}
 `))
