@@ -16,17 +16,15 @@ import (
 	"testing"
 
 	"github.com/google/go-cmp/cmp"
-	"github.com/google/go-cmp/cmp/cmpopts"
 	"golang.org/x/tools/gopls/internal/lsp/cache"
 	"golang.org/x/tools/gopls/internal/lsp/command"
+	"golang.org/x/tools/gopls/internal/lsp/debug"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/source"
 	"golang.org/x/tools/gopls/internal/lsp/tests"
 	"golang.org/x/tools/gopls/internal/lsp/tests/compare"
 	"golang.org/x/tools/gopls/internal/span"
 	"golang.org/x/tools/internal/bug"
-	"golang.org/x/tools/internal/diff"
-	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/testenv"
 )
 
@@ -34,13 +32,6 @@ func TestMain(m *testing.M) {
 	bug.PanicOnBugs = true
 	testenv.ExitIfSmallMachine()
 
-	// Set the global exporter to nil so that we don't log to stderr. This avoids
-	// a lot of misleading noise in test output.
-	//
-	// TODO(rfindley): investigate whether we can/should capture logs scoped to
-	// individual tests by passing in a context with a local exporter.
-	event.SetExporter(nil)
-
 	os.Exit(m.Run())
 }
 
@@ -54,6 +45,13 @@ func TestLSP(t *testing.T) {
 func testLSP(t *testing.T, datum *tests.Data) {
 	ctx := tests.Context(t)
 
+	// Setting a debug instance suppresses logging to stderr, but ensures that we
+	// still e.g. convert events into runtime/trace/instrumentation.
+	//
+	// Previously, we called event.SetExporter(nil), which turns off all
+	// instrumentation.
+	ctx = debug.WithInstance(ctx, "", "off")
+
 	session := cache.NewSession(ctx, cache.New(nil), nil)
 	options := source.DefaultOptions().Clone()
 	tests.DefaultOptions(options)
@@ -769,35 +767,6 @@ func (r *runner) Definition(t *testing.T, _ span.Span, d tests.Definition) {
 	}
 }
 
-func (r *runner) Implementation(t *testing.T, spn span.Span, wantSpans []span.Span) {
-	sm, err := r.data.Mapper(spn.URI())
-	if err != nil {
-		t.Fatal(err)
-	}
-	loc, err := sm.SpanLocation(spn)
-	if err != nil {
-		t.Fatal(err)
-	}
-	gotImpls, err := r.server.Implementation(r.ctx, &protocol.ImplementationParams{
-		TextDocumentPositionParams: protocol.LocationTextDocumentPositionParams(loc),
-	})
-	if err != nil {
-		t.Fatalf("Server.Implementation(%s): %v", spn, err)
-	}
-	gotLocs, err := tests.LocationsToSpans(r.data, gotImpls)
-	if err != nil {
-		t.Fatal(err)
-	}
-	sanitize := func(s string) string {
-		return strings.ReplaceAll(s, r.data.Config.Dir, "gopls/internal/lsp/testdata")
-	}
-	want := sanitize(tests.SortAndFormatSpans(wantSpans))
-	got := sanitize(tests.SortAndFormatSpans(gotLocs))
-	if got != want {
-		t.Errorf("implementations(%s):\n%s", sanitize(fmt.Sprint(spn)), diff.Unified("want", "got", want, got))
-	}
-}
-
 func (r *runner) Highlight(t *testing.T, src span.Span, spans []span.Span) {
 	m, err := r.data.Mapper(src.URI())
 	if err != nil {
@@ -842,90 +811,6 @@ func (r *runner) Highlight(t *testing.T, src span.Span, spans []span.Span) {
 	}
 }
 
-func (r *runner) References(t *testing.T, src span.Span, itemList []span.Span) {
-	// This test is substantially the same as (*runner).References in source/source_test.go.
-	// TODO(adonovan): Factor (and remove fluff). Where should the common code live?
-
-	sm, err := r.data.Mapper(src.URI())
-	if err != nil {
-		t.Fatal(err)
-	}
-	loc, err := sm.SpanLocation(src)
-	if err != nil {
-		t.Fatalf("failed for %v: %v", src, err)
-	}
-	for _, includeDeclaration := range []bool{true, false} {
-		t.Run(fmt.Sprintf("refs-declaration-%v", includeDeclaration), func(t *testing.T) {
-			want := make(map[protocol.Location]bool)
-			for i, pos := range itemList {
-				// We don't want the first result if we aren't including the declaration.
-				// TODO(adonovan): don't assume a single declaration:
-				// there may be >1 if corresponding methods are considered.
-				if i == 0 && !includeDeclaration {
-					continue
-				}
-				m, err := r.data.Mapper(pos.URI())
-				if err != nil {
-					t.Fatal(err)
-				}
-				loc, err := m.SpanLocation(pos)
-				if err != nil {
-					t.Fatalf("failed for %v: %v", src, err)
-				}
-				want[loc] = true
-			}
-			params := &protocol.ReferenceParams{
-				TextDocumentPositionParams: protocol.LocationTextDocumentPositionParams(loc),
-				Context: protocol.ReferenceContext{
-					IncludeDeclaration: includeDeclaration,
-				},
-			}
-			got, err := r.server.References(r.ctx, params)
-			if err != nil {
-				t.Fatalf("failed for %v: %v", src, err)
-			}
-
-			sanitize := func(s string) string {
-				// In practice, CONFIGDIR means "gopls/internal/lsp/testdata".
-				return strings.ReplaceAll(s, r.data.Config.Dir, "CONFIGDIR")
-			}
-			formatLocation := func(loc protocol.Location) string {
-				return fmt.Sprintf("%s:%d.%d-%d.%d",
-					sanitize(string(loc.URI)),
-					loc.Range.Start.Line+1,
-					loc.Range.Start.Character+1,
-					loc.Range.End.Line+1,
-					loc.Range.End.Character+1)
-			}
-			toSlice := func(set map[protocol.Location]bool) []protocol.Location {
-				// TODO(adonovan): use generic maps.Keys(), someday.
-				list := make([]protocol.Location, 0, len(set))
-				for key := range set {
-					list = append(list, key)
-				}
-				return list
-			}
-			toString := func(locs []protocol.Location) string {
-				// TODO(adonovan): use generic JoinValues(locs, formatLocation).
-				strs := make([]string, len(locs))
-				for i, loc := range locs {
-					strs[i] = formatLocation(loc)
-				}
-				sort.Strings(strs)
-				return strings.Join(strs, "\n")
-			}
-			gotStr := toString(got)
-			wantStr := toString(toSlice(want))
-			if gotStr != wantStr {
-				t.Errorf("incorrect references (got %d, want %d) at %s:\n%s",
-					len(got), len(want),
-					formatLocation(loc),
-					diff.Unified("want", "got", wantStr, gotStr))
-			}
-		})
-	}
-}
-
 func (r *runner) InlayHints(t *testing.T, spn span.Span) {
 	uri := spn.URI()
 	filename := uri.Filename()
@@ -1106,43 +991,6 @@ func applyTextDocumentEdits(r *runner, edits []protocol.DocumentChanges) (map[sp
 	return res, nil
 }
 
-func (r *runner) Symbols(t *testing.T, uri span.URI, expectedSymbols []protocol.DocumentSymbol) {
-	params := &protocol.DocumentSymbolParams{
-		TextDocument: protocol.TextDocumentIdentifier{
-			URI: protocol.URIFromSpanURI(uri),
-		},
-	}
-	got, err := r.server.DocumentSymbol(r.ctx, params)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	symbols := make([]protocol.DocumentSymbol, len(got))
-	for i, s := range got {
-		s, ok := s.(protocol.DocumentSymbol)
-		if !ok {
-			t.Fatalf("%v: wanted []DocumentSymbols but got %v", uri, got)
-		}
-		symbols[i] = s
-	}
-
-	// Sort by position to make it easier to find errors.
-	sortSymbols := func(s []protocol.DocumentSymbol) {
-		sort.Slice(s, func(i, j int) bool {
-			return protocol.CompareRange(s[i].SelectionRange, s[j].SelectionRange) < 0
-		})
-	}
-	sortSymbols(expectedSymbols)
-	sortSymbols(symbols)
-
-	// Ignore 'Range' here as it is difficult (impossible?) to express
-	// multi-line ranges in the packagestest framework.
-	ignoreRange := cmpopts.IgnoreFields(protocol.DocumentSymbol{}, "Range")
-	if diff := cmp.Diff(expectedSymbols, symbols, ignoreRange); diff != "" {
-		t.Errorf("mismatching symbols (-want +got)\n%s", diff)
-	}
-}
-
 func (r *runner) WorkspaceSymbols(t *testing.T, uri span.URI, query string, typ tests.WorkspaceSymbolsTestType) {
 	matcher := tests.WorkspaceSymbolsTestTypeToMatcher(typ)
 
@@ -1349,7 +1197,7 @@ func (r *runner) collectDiagnostics(view *cache.View) {
 	defer release()
 
 	// Always run diagnostics with analysis.
-	r.server.diagnose(r.ctx, snapshot, true)
+	r.server.diagnose(r.ctx, snapshot, analyzeEverything)
 	for uri, reports := range r.server.diagnostics {
 		for _, report := range reports.reports {
 			for _, d := range report.diags {
