@@ -8,10 +8,12 @@ import (
 	"bytes"
 	"context"
 	"encoding/json"
+	"errors"
 	"flag"
 	"fmt"
 	"go/token"
 	"io/fs"
+	"log"
 	"os"
 	"path"
 	"path/filepath"
@@ -21,6 +23,7 @@ import (
 	"strings"
 	"testing"
 
+	"github.com/google/go-cmp/cmp"
 	"golang.org/x/tools/go/expect"
 	"golang.org/x/tools/gopls/internal/hooks"
 	"golang.org/x/tools/gopls/internal/lsp/cache"
@@ -41,6 +44,11 @@ import (
 var update = flag.Bool("update", false, "if set, update test data during marker tests")
 
 // RunMarkerTests runs "marker" tests in the given test data directory.
+// (In practice: ../../regtest/marker/testdata)
+//
+// Use this command to run the tests:
+//
+//	$ go test ./gopls/internal/regtest/marker [-update]
 //
 // A marker test uses the '//@' marker syntax of the x/tools/go/expect package
 // to annotate source code with various information such as locations and
@@ -112,6 +120,9 @@ var update = flag.Bool("update", false, "if set, update test data during marker
 //
 // The following markers are supported within marker tests:
 //
+//   - complete(location, ...labels): specifies expected completion results at
+//     the given location.
+//
 //   - diag(location, regexp): specifies an expected diagnostic matching the
 //     given regexp at the given location. The test runner requires
 //     a 1:1 correspondence between observed diagnostics and diag annotations
@@ -123,6 +134,10 @@ var update = flag.Bool("update", false, "if set, update test data during marker
 //     src location, and checks that the result is the dst location, with hover
 //     content matching "hover.md" in the golden data g.
 //
+//   - implementations(src location, want ...location): makes a
+//     textDocument/implementation query at the src location and
+//     checks that the resulting set of locations matches want.
+//
 //   - loc(name, location): specifies the name for a location in the source. These
 //     locations may be referenced by other markers.
 //
@@ -139,6 +154,22 @@ var update = flag.Bool("update", false, "if set, update test data during marker
 //     This action is executed for its editing effects on the source files.
 //     Like rename, the golden directory contains the expected transformed files.
 //
+//   - refs(location, want ...location): executes a 'references' query at the
+//     first location and asserts that the result is the set of 'want' locations.
+//     The first want location must be the declaration (assumedly unique).
+//
+//   - symbol(golden): makes a textDocument/documentSymbol request
+//     for the enclosing file, formats the response with one symbol
+//     per line, sorts it, and compares against the named golden file.
+//     Each line is of the form:
+//
+//     dotted.symbol.name kind "detail" +n lines
+//
+//     where the "+n lines" part indicates that the declaration spans
+//     several lines. The test otherwise makes no attempt to check
+//     location information. There is no point to using more than one
+//     @symbol marker in a given file.
+//
 // # Argument conversion
 //
 // Marker arguments are first parsed by the go/expect package, which accepts
@@ -228,7 +259,7 @@ var update = flag.Bool("update", false, "if set, update test data during marker
 //   - reorganize regtest packages (and rename to just 'test'?)
 //   - Rename the files .txtar.
 //
-// Existing marker tests to port:
+// Existing marker tests (in ../testdata) to port:
 //   - CallHierarchy
 //   - CodeLens
 //   - Diagnostics
@@ -244,16 +275,11 @@ var update = flag.Bool("update", false, "if set, update test data during marker
 //   - Formats
 //   - Imports
 //   - SemanticTokens
-//   - SuggestedFixes
 //   - FunctionExtractions
 //   - MethodExtractions
-//   - Definitions
-//   - Implementations
 //   - Highlights
-//   - References
 //   - Renames
 //   - PrepareRenames
-//   - Symbols
 //   - InlayHints
 //   - WorkspaceSymbols
 //   - Signatures
@@ -282,12 +308,18 @@ func RunMarkerTests(t *testing.T, dir string) {
 				if _, err := fmt.Sscanf(test.minGoVersion, "go1.%d", &go1point); err != nil {
 					t.Fatalf("parsing -min_go version: %v", err)
 				}
-				testenv.NeedsGo1Point(t, 18)
+				testenv.NeedsGo1Point(t, go1point)
 			}
 			config := fake.EditorConfig{
 				Settings: test.settings,
 				Env:      test.env,
 			}
+			if _, ok := config.Settings["diagnosticsDelay"]; !ok {
+				if config.Settings == nil {
+					config.Settings = make(map[string]interface{})
+				}
+				config.Settings["diagnosticsDelay"] = "10ms"
+			}
 			run := &markerTestRun{
 				test: test,
 				env:  newEnv(t, cache, test.files, config),
@@ -372,6 +404,10 @@ func RunMarkerTests(t *testing.T, dir string) {
 			}
 		})
 	}
+
+	if abs, err := filepath.Abs(dir); err == nil && t.Failed() {
+		t.Logf("(Filenames are relative to %s.)", abs)
+	}
 }
 
 // A marker holds state for the execution of a single @marker
@@ -382,8 +418,10 @@ type marker struct {
 }
 
 // errorf reports an error with a prefix indicating the position of the marker note.
+//
+// It formats the error message using mark.sprintf.
 func (mark marker) errorf(format string, args ...interface{}) {
-	msg := fmt.Sprintf(format, args...)
+	msg := mark.sprintf(format, args...)
 	// TODO(adonovan): consider using fmt.Fprintf(os.Stderr)+t.Fail instead of
 	// t.Errorf to avoid reporting uninteresting positions in the Go source of
 	// the driver. However, this loses the order of stderr wrt "FAIL: TestFoo"
@@ -401,29 +439,39 @@ func (mark marker) execute() {
 
 	// The first converter corresponds to the *Env argument.
 	// All others must be converted from the marker syntax.
-	if got, want := len(mark.note.Args), len(fn.converters); got != want {
-		mark.errorf("got %d arguments to %s, expect %d", got, mark.note.Name, want)
-		return
-	}
-
 	args := []reflect.Value{reflect.ValueOf(mark)}
+	var convert converter
 	for i, in := range mark.note.Args {
-		// Special handling for the blank identifier: treat it as the zero
-		// value.
+		if i < len(fn.converters) {
+			convert = fn.converters[i]
+		} else if !fn.variadic {
+			goto arity // too many args
+		}
+
+		// Special handling for the blank identifier: treat it as the zero value.
 		if ident, ok := in.(expect.Identifier); ok && ident == "_" {
 			zero := reflect.Zero(fn.paramTypes[i])
 			args = append(args, zero)
 			continue
 		}
-		out, err := fn.converters[i](mark, in)
+
+		out, err := convert(mark, in)
 		if err != nil {
 			mark.errorf("converting argument #%d of %s (%v): %v", i, mark.note.Name, in, err)
 			return
 		}
 		args = append(args, reflect.ValueOf(out))
 	}
+	if len(args) < len(fn.converters) {
+		goto arity // too few args
+	}
 
 	fn.fn.Call(args)
+	return
+
+arity:
+	mark.errorf("got %d arguments to %s, want %d",
+		len(mark.note.Args), mark.note.Name, len(fn.converters))
 }
 
 // Supported marker functions.
@@ -434,13 +482,17 @@ func (mark marker) execute() {
 // Marker funcs should not mutate the test environment (e.g. via opening files
 // or applying edits in the editor).
 var markerFuncs = map[string]markerFunc{
-	"def":          makeMarkerFunc(defMarker),
-	"diag":         makeMarkerFunc(diagMarker),
-	"hover":        makeMarkerFunc(hoverMarker),
-	"loc":          makeMarkerFunc(locMarker),
-	"rename":       makeMarkerFunc(renameMarker),
-	"renameerr":    makeMarkerFunc(renameErrMarker),
-	"suggestedfix": makeMarkerFunc(suggestedfixMarker),
+	"complete":       makeMarkerFunc(completeMarker),
+	"def":            makeMarkerFunc(defMarker),
+	"diag":           makeMarkerFunc(diagMarker),
+	"hover":          makeMarkerFunc(hoverMarker),
+	"implementation": makeMarkerFunc(implementationMarker),
+	"loc":            makeMarkerFunc(locMarker),
+	"rename":         makeMarkerFunc(renameMarker),
+	"renameerr":      makeMarkerFunc(renameErrMarker),
+	"suggestedfix":   makeMarkerFunc(suggestedfixMarker),
+	"symbol":         makeMarkerFunc(symbolMarker),
+	"refs":           makeMarkerFunc(refsMarker),
 }
 
 // markerTest holds all the test data extracted from a test txtar archive.
@@ -562,6 +614,13 @@ func loadMarkerTests(dir string) ([]*markerTest, error) {
 
 func loadMarkerTest(name string, content []byte) (*markerTest, error) {
 	archive := txtar.Parse(content)
+	if len(archive.Files) == 0 {
+		return nil, fmt.Errorf("txtar file has no '-- filename --' sections")
+	}
+	if bytes.Contains(archive.Comment, []byte("\n-- ")) {
+		// This check is conservative, but the comment is only a comment.
+		return nil, fmt.Errorf("ill-formed '-- filename --' header in comment")
+	}
 	test := &markerTest{
 		name:    name,
 		fset:    token.NewFileSet(),
@@ -614,6 +673,13 @@ func loadMarkerTest(name string, content []byte) (*markerTest, error) {
 			test.notes = append(test.notes, notes...)
 			test.files[file.Name] = file.Data
 		}
+
+		// Print a warning if we see what looks like "-- filename --"
+		// without the second "--". It's not necessarily wrong,
+		// but it should almost never appear in our test inputs.
+		if bytes.Contains(file.Data, []byte("\n-- ")) {
+			log.Printf("ill-formed '-- filename --' header in %s?", file.Name)
+		}
 	}
 
 	return test, nil
@@ -720,6 +786,7 @@ type markerFunc struct {
 	fn         reflect.Value  // the func to invoke
 	paramTypes []reflect.Type // parameter types, for zero values
 	converters []converter    // to convert non-blank arguments
+	variadic   bool
 }
 
 // A markerTestRun holds the state of one run of a marker test archive.
@@ -733,6 +800,33 @@ type markerTestRun struct {
 	diags     map[protocol.Location][]protocol.Diagnostic
 }
 
+// sprintf returns a formatted string after applying pre-processing to
+// arguments of the following types:
+//   - token.Pos: formatted using (*markerTestRun).fmtPos
+//   - protocol.Location: formatted using (*markerTestRun).fmtLoc
+func (c *marker) sprintf(format string, args ...interface{}) string {
+	if false {
+		_ = fmt.Sprintf(format, args...) // enable vet printf checker
+	}
+	var args2 []interface{}
+	for _, arg := range args {
+		switch arg := arg.(type) {
+		case token.Pos:
+			args2 = append(args2, c.run.fmtPos(arg))
+		case protocol.Location:
+			args2 = append(args2, c.run.fmtLoc(arg))
+		default:
+			args2 = append(args2, arg)
+		}
+	}
+	return fmt.Sprintf(format, args2...)
+}
+
+// uri returns the URI of the file containing the marker.
+func (mark marker) uri() protocol.DocumentURI {
+	return mark.run.env.Sandbox.Workdir.URI(mark.run.test.fset.File(mark.note.Pos).Name())
+}
+
 // fmtLoc formats the given pos in the context of the test, using
 // archive-relative paths for files and including the line number in the full
 // archive file.
@@ -808,6 +902,7 @@ func makeMarkerFunc(fn interface{}) markerFunc {
 		fn: reflect.ValueOf(fn),
 	}
 	mtyp := mi.fn.Type()
+	mi.variadic = mtyp.IsVariadic()
 	if mtyp.NumIn() == 0 || mtyp.In(0) != markerType {
 		panic(fmt.Sprintf("marker function %#v must accept marker as its first argument", mi.fn))
 	}
@@ -816,6 +911,9 @@ func makeMarkerFunc(fn interface{}) markerFunc {
 	}
 	for a := 1; a < mtyp.NumIn(); a++ {
 		in := mtyp.In(a)
+		if mi.variadic && a == mtyp.NumIn()-1 {
+			in = in.Elem() // for ...T, convert to T
+		}
 		mi.paramTypes = append(mi.paramTypes, in)
 		c := makeConverter(in)
 		mi.converters = append(mi.converters, c)
@@ -1042,6 +1140,26 @@ func checkChangedFiles(mark marker, changed map[string][]byte, golden *Golden) {
 
 // ---- marker functions ----
 
+// completeMarker implements the @complete marker, running
+// textDocument/completion at the given src location and asserting that the
+// results match the expected results.
+//
+// TODO(rfindley): for now, this is just a quick check against the expected
+// completion labels. We could do more by assembling richer completion items,
+// as is done in the old marker tests. Does that add value? If so, perhaps we
+// should support a variant form of the argument, labelOrItem, which allows the
+// string form or item form.
+func completeMarker(mark marker, src protocol.Location, want ...string) {
+	list := mark.run.env.Completion(src)
+	var got []string
+	for _, item := range list.Items {
+		got = append(got, item.Label)
+	}
+	if diff := cmp.Diff(want, got); diff != "" {
+		mark.errorf("Completion(...) returned unexpect results (-want +got):\n%s", diff)
+	}
+}
+
 // defMarker implements the @godef marker, running textDocument/definition at
 // the given src location and asserting that there is exactly one resulting
 // location, matching dst.
@@ -1086,6 +1204,11 @@ func hoverMarker(mark marker, src, dst protocol.Location, golden *Golden) {
 // locMarker implements the @loc marker. It is executed before other
 // markers, so that locations are available.
 func locMarker(mark marker, name expect.Identifier, loc protocol.Location) {
+	if prev, dup := mark.run.locations[name]; dup {
+		mark.errorf("location %q already declared at %s",
+			name, mark.run.fmtLoc(prev))
+		return
+	}
 	mark.run.locations[name] = loc
 }
 
@@ -1105,7 +1228,7 @@ func removeDiagnostic(mark marker, loc protocol.Location, re *regexp.Regexp) (pr
 			return diag, nil
 		}
 	}
-	return protocol.Diagnostic{}, fmt.Errorf("no diagnostic matches %q", re)
+	return protocol.Diagnostic{}, errors.New(mark.sprintf("no diagnostic at %v matches %q", loc, re))
 }
 
 // renameMarker implements the @rename(location, new, golden) marker.
@@ -1271,3 +1394,135 @@ func suggestedfix(env *Env, loc protocol.Location, diag protocol.Diagnostic, act
 }
 
 // TODO(adonovan): suggestedfixerr
+
+// refsMarker implements the @refs marker.
+func refsMarker(mark marker, src protocol.Location, want ...protocol.Location) {
+	refs := func(includeDeclaration bool, want []protocol.Location) error {
+		got, err := mark.run.env.Editor.Server.References(mark.run.env.Ctx, &protocol.ReferenceParams{
+			TextDocumentPositionParams: protocol.LocationTextDocumentPositionParams(src),
+			Context: protocol.ReferenceContext{
+				IncludeDeclaration: includeDeclaration,
+			},
+		})
+		if err != nil {
+			return err
+		}
+
+		return compareLocations(mark, got, want)
+	}
+
+	for _, includeDeclaration := range []bool{false, true} {
+		// Ignore first 'want' location if we didn't request the declaration.
+		// TODO(adonovan): don't assume a single declaration:
+		// there may be >1 if corresponding methods are considered.
+		want := want
+		if !includeDeclaration && len(want) > 0 {
+			want = want[1:]
+		}
+		if err := refs(includeDeclaration, want); err != nil {
+			mark.errorf("refs(includeDeclaration=%t) failed: %v",
+				includeDeclaration, err)
+		}
+	}
+}
+
+// implementationMarker implements the @implementation marker.
+func implementationMarker(mark marker, src protocol.Location, want ...protocol.Location) {
+	got, err := mark.run.env.Editor.Server.Implementation(mark.run.env.Ctx, &protocol.ImplementationParams{
+		TextDocumentPositionParams: protocol.LocationTextDocumentPositionParams(src),
+	})
+	if err != nil {
+		mark.errorf("implementation at %s failed: %v", src, err)
+		return
+	}
+	if err := compareLocations(mark, got, want); err != nil {
+		mark.errorf("implementation: %v", err)
+	}
+}
+
+// symbolMarker implements the @symbol marker.
+func symbolMarker(mark marker, golden *Golden) {
+	// Retrieve information about all symbols in this file.
+	symbols, err := mark.run.env.Editor.Server.DocumentSymbol(mark.run.env.Ctx, &protocol.DocumentSymbolParams{
+		TextDocument: protocol.TextDocumentIdentifier{URI: mark.uri()},
+	})
+	if err != nil {
+		mark.errorf("DocumentSymbol request failed: %v", err)
+		return
+	}
+
+	// Format symbols one per line, sorted (in effect) by first column, a dotted name.
+	var lines []string
+	for _, symbol := range symbols {
+		// Each result element is a union of (legacy)
+		// SymbolInformation and (new) DocumentSymbol,
+		// so we ascertain which one and then transcode.
+		data, err := json.Marshal(symbol)
+		if err != nil {
+			mark.run.env.T.Fatal(err)
+		}
+		if _, ok := symbol.(map[string]interface{})["location"]; ok {
+			// This case is not reached because Editor initialization
+			// enables HierarchicalDocumentSymbolSupport.
+			// TODO(adonovan): test this too.
+			var sym protocol.SymbolInformation
+			if err := json.Unmarshal(data, &sym); err != nil {
+				mark.run.env.T.Fatal(err)
+			}
+			mark.errorf("fake Editor doesn't support SymbolInformation")
+
+		} else {
+			var sym protocol.DocumentSymbol // new hierarchical hotness
+			if err := json.Unmarshal(data, &sym); err != nil {
+				mark.run.env.T.Fatal(err)
+			}
+
+			// Print each symbol in the response tree.
+			var visit func(sym protocol.DocumentSymbol, prefix []string)
+			visit = func(sym protocol.DocumentSymbol, prefix []string) {
+				var out strings.Builder
+				out.WriteString(strings.Join(prefix, "."))
+				fmt.Fprintf(&out, " %q", sym.Detail)
+				if delta := sym.Range.End.Line - sym.Range.Start.Line; delta > 0 {
+					fmt.Fprintf(&out, " +%d lines", delta)
+				}
+				lines = append(lines, out.String())
+
+				for _, child := range sym.Children {
+					visit(child, append(prefix, child.Name))
+				}
+			}
+			visit(sym, []string{sym.Name})
+		}
+	}
+	sort.Strings(lines)
+	lines = append(lines, "") // match trailing newline in .txtar file
+	got := []byte(strings.Join(lines, "\n"))
+
+	// Compare with golden.
+	want, ok := golden.Get(mark.run.env.T, "", got)
+	if !ok {
+		mark.errorf("%s: missing golden file @%s", mark.note.Name, golden.id)
+	} else if diff := cmp.Diff(string(got), string(want)); diff != "" {
+		mark.errorf("%s: unexpected output: got:\n%s\nwant:\n%s\ndiff:\n%s",
+			mark.note.Name, got, want, diff)
+	}
+}
+
+// compareLocations returns an error message if got and want are not
+// the same set of locations. The marker is used only for fmtLoc.
+func compareLocations(mark marker, got, want []protocol.Location) error {
+	toStrings := func(locs []protocol.Location) []string {
+		strs := make([]string, len(locs))
+		for i, loc := range locs {
+			strs[i] = mark.run.fmtLoc(loc)
+		}
+		sort.Strings(strs)
+		return strs
+	}
+	if diff := cmp.Diff(toStrings(want), toStrings(got)); diff != "" {
+		return fmt.Errorf("incorrect result locations: (got %d, want %d):\n%s",
+			len(got), len(want), diff)
+	}
+	return nil
+}
