@@ -11,6 +11,7 @@ import (
 	"errors"
 	"fmt"
 	"go/ast"
+	"go/parser"
 	"go/scanner"
 	"go/token"
 	"go/types"
@@ -67,14 +68,16 @@ type Snapshot interface {
 	// and their GOPATH.
 	ValidBuildConfiguration() bool
 
+	// A Snapshot is a caching implementation of FileSource whose
+	// ReadFile method returns consistent information about the existence
+	// and content of each file throughout its lifetime.
+	FileSource
+
 	// FindFile returns the FileHandle for the given URI, if it is already
 	// in the given snapshot.
+	// TODO(adonovan): delete this operation; use ReadFile instead.
 	FindFile(uri span.URI) FileHandle
 
-	// GetFile returns the FileHandle for a given URI, initializing it if it is
-	// not already part of the snapshot.
-	GetFile(ctx context.Context, uri span.URI) (FileHandle, error)
-
 	// AwaitInitialized waits until the snapshot's view is initialized.
 	AwaitInitialized(ctx context.Context)
 
@@ -91,7 +94,7 @@ type Snapshot interface {
 	// ParseGo returns the parsed AST for the file.
 	// If the file is not available, returns nil and an error.
 	// Position information is added to FileSet().
-	ParseGo(ctx context.Context, fh FileHandle, mode ParseMode) (*ParsedGoFile, error)
+	ParseGo(ctx context.Context, fh FileHandle, mode parser.Mode) (*ParsedGoFile, error)
 
 	// Analyze runs the specified analyzers on the given package at this snapshot.
 	Analyze(ctx context.Context, id PackageID, analyzers []*Analyzer) ([]*Diagnostic, error)
@@ -363,10 +366,11 @@ type View interface {
 	GoVersionString() string
 }
 
-// A FileSource maps uris to FileHandles.
+// A FileSource maps URIs to FileHandles.
 type FileSource interface {
-	// GetFile returns the FileHandle for a given URI.
-	GetFile(ctx context.Context, uri span.URI) (FileHandle, error)
+	// ReadFile returns the FileHandle for a given URI, either by
+	// reading the content of the file or by obtaining it from a cache.
+	ReadFile(ctx context.Context, uri span.URI) (FileHandle, error)
 }
 
 // A MetadataSource maps package IDs to metadata.
@@ -382,17 +386,39 @@ type MetadataSource interface {
 // A ParsedGoFile contains the results of parsing a Go file.
 type ParsedGoFile struct {
 	URI  span.URI
-	Mode ParseMode
+	Mode parser.Mode
 	File *ast.File
 	Tok  *token.File
 	// Source code used to build the AST. It may be different from the
 	// actual content of the file if we have fixed the AST.
-	Src      []byte
-	Fixed    bool
+	Src []byte
+
+	// FixedSrc and Fixed AST report on "fixing" that occurred during parsing of
+	// this file.
+	//
+	// If FixedSrc == true, the source contained in the Src field was modified
+	// from the original source to improve parsing.
+	//
+	// If FixedAST == true, the ast was modified after parsing, and therefore
+	// positions encoded in the AST may not accurately represent the content of
+	// the Src field.
+	//
+	// TODO(rfindley): there are many places where we haphazardly use the Src or
+	// positions without checking these fields. Audit these places and guard
+	// accordingly. After doing so, we may find that we don't need to
+	// differentiate FixedSrc and FixedAST.
+	FixedSrc bool
+	FixedAST bool
 	Mapper   *protocol.Mapper // may map fixed Src, not file content
 	ParseErr scanner.ErrorList
 }
 
+// Fixed reports whether p was "Fixed", meaning that its source or positions
+// may not correlate with the original file.
+func (p ParsedGoFile) Fixed() bool {
+	return p.FixedSrc || p.FixedAST
+}
+
 // -- go/token domain convenience helpers --
 
 // PositionPos returns the token.Pos of protocol position p within the file.
@@ -595,22 +621,26 @@ func IsNonFatalGoModError(err error) bool {
 	return err == ErrTmpModfileUnsupported || err == ErrNoModOnDisk
 }
 
-// ParseMode controls the content of the AST produced when parsing a source file.
-type ParseMode int
-
+// Common parse modes; these should be reused wherever possible to increase
+// cache hits.
 const (
 	// ParseHeader specifies that the main package declaration and imports are needed.
 	// This is the mode used when attempting to examine the package graph structure.
-	ParseHeader ParseMode = iota
+	ParseHeader = parser.AllErrors | parser.ParseComments | parser.ImportsOnly | SkipObjectResolution
 
 	// ParseFull specifies the full AST is needed.
 	// This is used for files of direct interest where the entire contents must
 	// be considered.
-	ParseFull
+	ParseFull = parser.AllErrors | parser.ParseComments | SkipObjectResolution
 )
 
-// A FileHandle is an interface to files tracked by the LSP session, which may
-// be either files read from disk, or open in the editor session (overlays).
+// A FileHandle represents the URI, content, hash, and optional
+// version of a file tracked by the LSP session.
+//
+// File content may be provided by the file system (for Saved files)
+// or from an overlay, for open files with unsaved edits.
+// A FileHandle may record an attempt to read a non-existent file,
+// in which case Content returns an error.
 type FileHandle interface {
 	// URI is the URI for this file handle.
 	// TODO(rfindley): this is not actually well-defined. In some cases, there
@@ -620,15 +650,15 @@ type FileHandle interface {
 	// FileIdentity returns a FileIdentity for the file, even if there was an
 	// error reading it.
 	FileIdentity() FileIdentity
-	// Saved reports whether the file has the same content on disk.
-	// For on-disk files, this is trivially true.
+	// Saved reports whether the file has the same content on disk:
+	// it is false for files open on an editor with unsaved edits.
 	Saved() bool
 	// Version returns the file version, as defined by the LSP client.
 	// For on-disk file handles, Version returns 0.
 	Version() int32
-	// Read reads the contents of a file.
+	// Content returns the contents of a file.
 	// If the file is not available, returns a nil slice and an error.
-	Read() ([]byte, error)
+	Content() ([]byte, error)
 }
 
 // A Hash is a cryptographic digest of the contents of a file.
@@ -782,7 +812,6 @@ type Package interface {
 
 	// Results of parsing:
 	FileSet() *token.FileSet
-	ParseMode() ParseMode
 	CompiledGoFiles() []*ParsedGoFile // (borrowed)
 	File(uri span.URI) (*ParsedGoFile, error)
 	GetSyntax() []*ast.File // (borrowed)
@@ -812,7 +841,7 @@ type CriticalError struct {
 // An Diagnostic corresponds to an LSP Diagnostic.
 // https://microsoft.github.io/language-server-protocol/specification#diagnostic
 type Diagnostic struct {
-	URI      span.URI
+	URI      span.URI // of diagnosed file (not diagnostic documentation)
 	Range    protocol.Range
 	Severity protocol.DiagnosticSeverity
 	Code     string
@@ -850,6 +879,7 @@ const (
 	Govulncheck              DiagnosticSource = "govulncheck"
 	TemplateError            DiagnosticSource = "template"
 	WorkFileError            DiagnosticSource = "go.work file"
+	ConsistencyInfo          DiagnosticSource = "consistency"
 )
 
 func AnalyzerErrorKind(name string) DiagnosticSource {
