@@ -82,12 +82,9 @@ type SuggestedFixes = map[span.Span][]SuggestedFix
 type FunctionExtractions = map[span.Span]span.Span
 type MethodExtractions = map[span.Span]span.Span
 type Definitions = map[span.Span]Definition
-type Implementations = map[span.Span][]span.Span
 type Highlights = map[span.Span][]span.Span
-type References = map[span.Span][]span.Span
 type Renames = map[span.Span]string
 type PrepareRenames = map[span.Span]*source.PrepareItem
-type Symbols = map[span.URI][]*symbol
 type InlayHints = []span.Span
 type WorkspaceSymbols = map[WorkspaceSymbolsTestType]map[span.URI][]string
 type Signatures = map[span.Span]*protocol.SignatureHelp
@@ -117,13 +114,10 @@ type Data struct {
 	FunctionExtractions      FunctionExtractions
 	MethodExtractions        MethodExtractions
 	Definitions              Definitions
-	Implementations          Implementations
 	Highlights               Highlights
-	References               References
 	Renames                  Renames
 	InlayHints               InlayHints
 	PrepareRenames           PrepareRenames
-	Symbols                  Symbols
 	WorkspaceSymbols         WorkspaceSymbols
 	Signatures               Signatures
 	Links                    Links
@@ -166,13 +160,10 @@ type Tests interface {
 	FunctionExtraction(*testing.T, span.Span, span.Span)
 	MethodExtraction(*testing.T, span.Span, span.Span)
 	Definition(*testing.T, span.Span, Definition)
-	Implementation(*testing.T, span.Span, []span.Span)
 	Highlight(*testing.T, span.Span, []span.Span)
 	InlayHints(*testing.T, span.Span)
-	References(*testing.T, span.Span, []span.Span)
 	Rename(*testing.T, span.Span, string)
 	PrepareRename(*testing.T, span.Span, *source.PrepareItem)
-	Symbols(*testing.T, span.URI, []protocol.DocumentSymbol)
 	WorkspaceSymbols(*testing.T, span.URI, string, WorkspaceSymbolsTestType)
 	SignatureHelp(*testing.T, span.Span, *protocol.SignatureHelp)
 	Link(*testing.T, span.URI, []Link)
@@ -246,12 +237,6 @@ type SuggestedFix struct {
 	ActionKind, Title string
 }
 
-// A symbol holds a DocumentSymbol along with its parent-child edge.
-type symbol struct {
-	pSymbol      protocol.DocumentSymbol
-	id, parentID string
-}
-
 type Golden struct {
 	Filename string
 	Archive  *txtar.Archive
@@ -316,15 +301,12 @@ func load(t testing.TB, mode string, dir string) *Data {
 		RankCompletions:          make(RankCompletions),
 		CaseSensitiveCompletions: make(CaseSensitiveCompletions),
 		Definitions:              make(Definitions),
-		Implementations:          make(Implementations),
 		Highlights:               make(Highlights),
-		References:               make(References),
 		Renames:                  make(Renames),
 		PrepareRenames:           make(PrepareRenames),
 		SuggestedFixes:           make(SuggestedFixes),
 		FunctionExtractions:      make(FunctionExtractions),
 		MethodExtractions:        make(MethodExtractions),
-		Symbols:                  make(Symbols),
 		WorkspaceSymbols:         make(WorkspaceSymbols),
 		Signatures:               make(Signatures),
 		Links:                    make(Links),
@@ -461,39 +443,36 @@ func load(t testing.TB, mode string, dir string) *Data {
 
 	// Collect any data that needs to be used by subsequent tests.
 	if err := datum.Exported.Expect(map[string]interface{}{
-		"codelens":        datum.collectCodeLens,
-		"diag":            datum.collectDiagnostics,
-		"item":            datum.collectCompletionItems,
-		"complete":        datum.collectCompletions(CompletionDefault),
-		"unimported":      datum.collectCompletions(CompletionUnimported),
-		"deep":            datum.collectCompletions(CompletionDeep),
-		"fuzzy":           datum.collectCompletions(CompletionFuzzy),
-		"casesensitive":   datum.collectCompletions(CompletionCaseSensitive),
-		"rank":            datum.collectCompletions(CompletionRank),
-		"snippet":         datum.collectCompletionSnippets,
-		"fold":            datum.collectFoldingRanges,
-		"format":          datum.collectFormats,
-		"import":          datum.collectImports,
-		"semantic":        datum.collectSemanticTokens,
-		"godef":           datum.collectDefinitions,
-		"implementations": datum.collectImplementations,
-		"typdef":          datum.collectTypeDefinitions,
-		"hoverdef":        datum.collectHoverDefinitions,
-		"highlight":       datum.collectHighlights,
-		"inlayHint":       datum.collectInlayHints,
-		"refs":            datum.collectReferences,
-		"rename":          datum.collectRenames,
-		"prepare":         datum.collectPrepareRenames,
-		"symbol":          datum.collectSymbols,
-		"signature":       datum.collectSignatures,
-		"link":            datum.collectLinks,
-		"suggestedfix":    datum.collectSuggestedFixes,
-		"extractfunc":     datum.collectFunctionExtractions,
-		"extractmethod":   datum.collectMethodExtractions,
-		"incomingcalls":   datum.collectIncomingCalls,
-		"outgoingcalls":   datum.collectOutgoingCalls,
-		"addimport":       datum.collectAddImports,
-		"selectionrange":  datum.collectSelectionRanges,
+		"codelens":       datum.collectCodeLens,
+		"diag":           datum.collectDiagnostics,
+		"item":           datum.collectCompletionItems,
+		"complete":       datum.collectCompletions(CompletionDefault),
+		"unimported":     datum.collectCompletions(CompletionUnimported),
+		"deep":           datum.collectCompletions(CompletionDeep),
+		"fuzzy":          datum.collectCompletions(CompletionFuzzy),
+		"casesensitive":  datum.collectCompletions(CompletionCaseSensitive),
+		"rank":           datum.collectCompletions(CompletionRank),
+		"snippet":        datum.collectCompletionSnippets,
+		"fold":           datum.collectFoldingRanges,
+		"format":         datum.collectFormats,
+		"import":         datum.collectImports,
+		"semantic":       datum.collectSemanticTokens,
+		"godef":          datum.collectDefinitions,
+		"typdef":         datum.collectTypeDefinitions,
+		"hoverdef":       datum.collectHoverDefinitions,
+		"highlight":      datum.collectHighlights,
+		"inlayHint":      datum.collectInlayHints,
+		"rename":         datum.collectRenames,
+		"prepare":        datum.collectPrepareRenames,
+		"signature":      datum.collectSignatures,
+		"link":           datum.collectLinks,
+		"suggestedfix":   datum.collectSuggestedFixes,
+		"extractfunc":    datum.collectFunctionExtractions,
+		"extractmethod":  datum.collectMethodExtractions,
+		"incomingcalls":  datum.collectIncomingCalls,
+		"outgoingcalls":  datum.collectOutgoingCalls,
+		"addimport":      datum.collectAddImports,
+		"selectionrange": datum.collectSelectionRanges,
 	}); err != nil {
 		t.Fatal(err)
 	}
@@ -757,16 +736,6 @@ func Run(t *testing.T, tests Tests, data *Data) {
 		}
 	})
 
-	t.Run("Implementation", func(t *testing.T) {
-		t.Helper()
-		for spn, m := range data.Implementations {
-			t.Run(SpanName(spn), func(t *testing.T) {
-				t.Helper()
-				tests.Implementation(t, spn, m)
-			})
-		}
-	})
-
 	t.Run("Highlight", func(t *testing.T) {
 		t.Helper()
 		for pos, locations := range data.Highlights {
@@ -787,16 +756,6 @@ func Run(t *testing.T, tests Tests, data *Data) {
 		}
 	})
 
-	t.Run("References", func(t *testing.T) {
-		t.Helper()
-		for src, itemList := range data.References {
-			t.Run(SpanName(src), func(t *testing.T) {
-				t.Helper()
-				tests.References(t, src, itemList)
-			})
-		}
-	})
-
 	t.Run("Renames", func(t *testing.T) {
 		t.Helper()
 		for spn, newText := range data.Renames {
@@ -817,50 +776,6 @@ func Run(t *testing.T, tests Tests, data *Data) {
 		}
 	})
 
-	t.Run("Symbols", func(t *testing.T) {
-		t.Helper()
-		for uri, allSymbols := range data.Symbols {
-			byParent := make(map[string][]*symbol)
-			for _, sym := range allSymbols {
-				if sym.parentID != "" {
-					byParent[sym.parentID] = append(byParent[sym.parentID], sym)
-				}
-			}
-
-			// collectChildren does a depth-first traversal of the symbol tree,
-			// computing children of child nodes before returning to their parent.
-			// This is necessary as the Children field is slice of non-pointer types,
-			// and therefore we need to be careful to mutate children first before
-			// assigning them to their parent.
-			var collectChildren func(id string) []protocol.DocumentSymbol
-			collectChildren = func(id string) []protocol.DocumentSymbol {
-				children := byParent[id]
-				// delete from byParent before recursing, to ensure that
-				// collectChildren terminates even in the presence of cycles.
-				delete(byParent, id)
-				var result []protocol.DocumentSymbol
-				for _, child := range children {
-					child.pSymbol.Children = collectChildren(child.id)
-					result = append(result, child.pSymbol)
-				}
-				return result
-			}
-
-			var topLevel []protocol.DocumentSymbol
-			for _, sym := range allSymbols {
-				if sym.parentID == "" {
-					sym.pSymbol.Children = collectChildren(sym.id)
-					topLevel = append(topLevel, sym.pSymbol)
-				}
-			}
-
-			t.Run(uriName(uri), func(t *testing.T) {
-				t.Helper()
-				tests.Symbols(t, uri, topLevel)
-			})
-		}
-	})
-
 	t.Run("WorkspaceSymbols", func(t *testing.T) {
 		t.Helper()
 
@@ -1019,14 +934,11 @@ func checkData(t *testing.T, data *Data) {
 	fmt.Fprintf(buf, "TypeDefinitionsCount = %v\n", typeDefinitionCount)
 	fmt.Fprintf(buf, "HighlightsCount = %v\n", len(data.Highlights))
 	fmt.Fprintf(buf, "InlayHintsCount = %v\n", len(data.InlayHints))
-	fmt.Fprintf(buf, "ReferencesCount = %v\n", len(data.References))
 	fmt.Fprintf(buf, "RenamesCount = %v\n", len(data.Renames))
 	fmt.Fprintf(buf, "PrepareRenamesCount = %v\n", len(data.PrepareRenames))
-	fmt.Fprintf(buf, "SymbolsCount = %v\n", len(data.Symbols))
 	fmt.Fprintf(buf, "WorkspaceSymbolsCount = %v\n", countWorkspaceSymbols(data.WorkspaceSymbols))
 	fmt.Fprintf(buf, "SignaturesCount = %v\n", len(data.Signatures))
 	fmt.Fprintf(buf, "LinksCount = %v\n", linksCount)
-	fmt.Fprintf(buf, "ImplementationsCount = %v\n", len(data.Implementations))
 	fmt.Fprintf(buf, "SelectionRangesCount = %v\n", len(data.SelectionRanges))
 
 	want := string(data.Golden(t, "summary", summaryFile, func() ([]byte, error) {
@@ -1233,10 +1145,6 @@ func (data *Data) collectSelectionRanges(spn span.Span) {
 	data.SelectionRanges = append(data.SelectionRanges, spn)
 }
 
-func (data *Data) collectImplementations(src span.Span, targets []span.Span) {
-	data.Implementations[src] = targets
-}
-
 func (data *Data) collectIncomingCalls(src span.Span, calls []span.Span) {
 	for _, call := range calls {
 		rng := data.mustRange(call)
@@ -1302,10 +1210,6 @@ func (data *Data) collectInlayHints(src span.Span) {
 	data.InlayHints = append(data.InlayHints, src)
 }
 
-func (data *Data) collectReferences(src span.Span, expected []span.Span) {
-	data.References[src] = expected
-}
-
 func (data *Data) collectRenames(src span.Span, newText string) {
 	data.Renames[src] = newText
 }
@@ -1317,23 +1221,6 @@ func (data *Data) collectPrepareRenames(src, spn span.Span, placeholder string)
 	}
 }
 
-// collectSymbols is responsible for collecting @symbol annotations.
-func (data *Data) collectSymbols(name string, selectionRng span.Span, kind, detail, id, parentID string) {
-	// We don't set 'Range' here as it is difficult (impossible?) to express
-	// multi-line ranges in the packagestest framework.
-	uri := selectionRng.URI()
-	data.Symbols[uri] = append(data.Symbols[uri], &symbol{
-		pSymbol: protocol.DocumentSymbol{
-			Name:           name,
-			Kind:           protocol.ParseSymbolKind(kind),
-			SelectionRange: data.mustRange(selectionRng),
-			Detail:         detail,
-		},
-		id:       id,
-		parentID: parentID,
-	})
-}
-
 // mustRange converts spn into a protocol.Range, panicking on any error.
 func (data *Data) mustRange(spn span.Span) protocol.Range {
 	m, err := data.Mapper(spn.URI())
